/*
 * PmodBT2.h
 *
 *  Created on: Nov 8, 2017
 *      Author: Krzysztof Chojnowski
 */

#ifndef PMODBT2_H_
#define PMODBT2_H_

#include <stdbool.h>

#include "stm32l4xx_hal.h"

/**
 * @brief Configure ARDUINO CONNECTOR pins. This configuration uses two transmission pins (RX, TX),
 *        two hardware flow control pins (RTS, CTS), Status pin as an interrupt source and ~RST
 *        pin for module reset.
 *        PB13 -> RTS
 *        PB10 -> RX
 *        PB11 -> TX
 *        PB14 -> CTS
 *        PB12 -> STATUS
 *        PB15 -> ~RST
 */
void PmodBT2_Config(void);

/**
 * @brief Enables clocks required by USART3 and GPIO, initializes USART3 GPIO pins in alternate mode
 *        and enables UART interrupts. This function is required by HAL library and needs to be
 *        exported because two UART ports are used in this example.
 */
void PmodBT2_HAL_UART_MspInit(UART_HandleTypeDef *huart);

/**
 * @brief Reads version information from the RN-42 chip used in PmodBT2 module. It uses blocking
 *        UART read and write transaction to enter the command mode, read data and exit command mode.
 *        The RN-42 responses are checked and in case of error the function returns false. Otherwise
 *        the obtained data is stored in the given buffer and message length is written to the given
 *        pointer. Only the first line (ending with newline character) is returned.
 */
bool PmodBT2_ReadVersion(char* data, int* len);

/**
 * @brief Sends data to USART3 port HAL driver in non-blocking mode. If the interrupt handling routine
 *        is defined, it will be called after the transfer ends. In this example the USART3 interrupt
 *        routines are defined in the main.c file.
 */
void PmodBT2_Write(char* data, int len);

/**
 * @brief Enables a non-blocking read operation from the USART3 port. If the interrupt handling routine
 *        is defined, it will be called after given number of bytes is received. In this example
 *        the USART3 interrupt routines are defined in the main.c file.
 */
void PmodBT2_Read(char* data, int len);

#endif /* PMODBT2_H_ */
