/*
 * PmodBT2.c
 *
 *  Created on: Nov 8, 2017
 *      Author: Krzysztof Chojnowski
 */

#include <string.h>

#include "PmodBT2.h"

// The handle to the UART port used for the communication with the PmodBT2 module.
UART_HandleTypeDef pmodBt2Uart;

void PmodBT2_Config(void)
{
  // Enable clock for all required GPIO ports.
  __HAL_RCC_GPIOB_CLK_ENABLE();

  // Configure the interrupt on PB12 (PmodBT2 Status line). The interrupt is active on both edges
  // to indicate the connection and disconnection events.
  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Pin = GPIO_PIN_12;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  HAL_NVIC_SetPriority(EXTI15_10_IRQn, 2, 0);
  HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

  // Initialize the reset pin (PB15)
  GPIO_InitStruct.Mode  = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull  = GPIO_PULLUP;
  GPIO_InitStruct.Pin = GPIO_PIN_15;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  // Perform the RN-42 chip reset. The delays are introduced to make sure that the device will
  // have enough time for proper initialization.
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_15, GPIO_PIN_RESET);
  HAL_Delay(500);
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_15, GPIO_PIN_SET);
  HAL_Delay(500);

  // Initialize the USART3 port for RN-42 communication via Pmod connector.
  pmodBt2Uart.Instance = USART3;
  pmodBt2Uart.Init.BaudRate = 115200;
  pmodBt2Uart.Init.WordLength = UART_WORDLENGTH_8B;
  pmodBt2Uart.Init.StopBits = UART_STOPBITS_1;
  pmodBt2Uart.Init.Parity = UART_PARITY_NONE;
  pmodBt2Uart.Init.Mode = UART_MODE_TX_RX;
  pmodBt2Uart.Init.HwFlowCtl = UART_HWCONTROL_RTS_CTS;
  pmodBt2Uart.Init.OverSampling = UART_OVERSAMPLING_16;
  pmodBt2Uart.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;

  HAL_UART_Init(&pmodBt2Uart);
}

void PmodBT2_HAL_UART_MspInit(UART_HandleTypeDef *huart)
{
  // Enable the clocks for GPIO pins used by the USART3 port (PB10, PB11, PB13, PB1014).
  __HAL_RCC_USART3_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF7_USART3;
  GPIO_InitStruct.Pin = GPIO_PIN_10 | GPIO_PIN_11 | GPIO_PIN_13 | GPIO_PIN_14;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  // Enable the interrupts generated by USART3 port.
  HAL_NVIC_SetPriority(USART3_IRQn, 1, 0);
  HAL_NVIC_EnableIRQ(USART3_IRQn);
}

bool PmodBT2_ReadVersion(char* data, int* len)
{
  // Local buffer to store and check the RN-42 chip responses during entering
  // and exiting the command mode.
  char cmdResponse[5];

  // Enter the command mode. If the response is incorrect return false.
  HAL_UART_Transmit(&pmodBt2Uart, (uint8_t*)"$$$", 3, 100);
  HAL_UART_Receive(&pmodBt2Uart, (uint8_t*)cmdResponse, 5, 100);
  if(strncmp(cmdResponse, "CMD\r\n", 5) != 0)
    return false;

  // Read the version information to the given data buffer.
  HAL_UART_Transmit(&pmodBt2Uart, (uint8_t*)"V\r", 2, 100);
  HAL_UART_Receive(&pmodBt2Uart, (uint8_t*)data, 100, 100);

  // Exit the command mode. If the response is incorrect return false.
  HAL_UART_Transmit(&pmodBt2Uart, (uint8_t*)"---\r", 4, 100);
  HAL_UART_Receive(&pmodBt2Uart, (uint8_t*)cmdResponse, 5, 100);
  if(strncmp(cmdResponse, "END\r\n", 5) != 0)
      return false;

  // Return the length of the first line of the version information.
  *len = 0;
  while(data[(*len)++] != '\n');

  return true;
}

void PmodBT2_Write(char* data, int len)
{
  // Non-blocking data transfer.
  HAL_UART_Transmit_IT(&pmodBt2Uart, (uint8_t*)data, len);
}

void PmodBT2_Read(char* data, int len)
{
  // Non-blocking data reception.
  HAL_UART_Receive_IT(&pmodBt2Uart, (uint8_t*)data, len);
}

