/*
******************************************************************************
File:     main.c
Info:     Generated by Atollic TrueSTUDIO(R) 8.0.0   2017-10-28

The MIT License (MIT)
Copyright (c) 2009-2017 Atollic AB

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

******************************************************************************
*/

#include <string.h>

#include "stm32l4xx.h"
#include "stm32l4xx_hal.h"
#include "system_clock.h"
#include "serial.h"
#include "led.h"
#include "PmodBT2.h"

// The message sent before every response from the example application.
static char* echoText = "PmodBT2 ECHO: ";
static int echoTextLen = 14;

// The data buffer used in non-blocking data transfers handled by the interrupts
// and during version reading.
static char textBuffer[128];
static int bufferIndex = 0;

int main(void)
{
  HAL_Init();
  SystemClock_Config();
  Led_Config();
  Serial_Config();

  PmodBT2_Config();

  // Read the version information and write them to the serial port (USART1). In case of
  // failure, write the error information.
  int len;
  if(PmodBT2_ReadVersion(textBuffer, &len))
    Serial_Write(textBuffer, len);
  else
    Serial_Write("Version reading failed\n", 23);

  // Reset the data buffer index and start the non-blocking data reading. The interrupt
  // will be called after the first received byte.
  bufferIndex = 0;
  PmodBT2_Read(textBuffer, 1);

  while(1);
}

/**
 * @brief General UART configuration function called by the HAL library during each port
 *        initialization. This function calls the MspInit function of the appropriate driver.
 */
void HAL_UART_MspInit(UART_HandleTypeDef *huart)
{
  if(huart->Instance == USART3)
    PmodBT2_HAL_UART_MspInit(huart);
  else if(huart->Instance == LPUART1)
    Serial_HAL_UART_MspInit(huart);
}
/**
 * @brief Data reception interrupt handler. The interrupt is configured by the PmodBT2
 *        driver (USART3).
 */
void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart)
{
  // Wait for the line end and write the echo message. If the lastly obtained byte is not
  // the newline character, wait for the next one. After obtaining the newline character
  // write the defined echo message.
  if(textBuffer[bufferIndex] == '\n')
    PmodBT2_Write(echoText, echoTextLen);
  else {
    bufferIndex++;
    PmodBT2_Read(&textBuffer[bufferIndex], 1);
  }
}

/**
 * @brief Data transmission finished interrupt handler. The interrupt is configured
 *        by the PmodBT2 driver (USART3).
 */
void HAL_UART_TxCpltCallback(UART_HandleTypeDef *huart)
{
  // Check whether there is something to send.
  if(bufferIndex == 0)
  {
	// Schedule non-blocking data reception.
	PmodBT2_Read(textBuffer, 1);
    return;
  }

  // Send all received and buffered data and clear the buffer index.
  PmodBT2_Write(textBuffer, bufferIndex + 1);
  bufferIndex = 0;
}

/**
 * @brief GPIO interrupt handler. The interrupt is configured by the PmodBT2 driver (PD2)
 *        and indicates the connection state change.
 */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
  // The high level of the Status pin indicates than the connection is active.
  if(HAL_GPIO_ReadPin(GPIOB, GPIO_Pin) == GPIO_PIN_SET)
    Led_TurnOn(LED1);
  else
    Led_TurnOff(LED1);
}
